package com.alihealth.d2d.provtest.service.impl;

import com.alihealth.d2d.provtest.VO.*;
import com.alihealth.d2d.provtest.exception.ServiceException;
import com.alihealth.d2d.provtest.mapper.*;
import com.alihealth.d2d.provtest.searcher.DrugRetraceSearcher;
import com.alihealth.d2d.provtest.service.DrugRetraceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Service("drugRetraceService")
public class DrugRetraceServiceImpl implements DrugRetraceService {

    @Autowired
    private DomesticDrugProductionApplyInfoMapper domesticDrugProductionApplyInfoMapper;

    @Autowired
    private DrugImportApplyInfoMapper drugImportApplyInfoMapper;

    @Autowired
    private DrugInvoiceApplyInfoMapper drugInvoiceApplyInfoMapper;

    @Autowired
    private DrugDeliveryNoteApplyInfoMapper drugDeliveryNoteApplyInfoMapper;

    @Autowired
    private DrugRetailAndUseApplyInfoMapper drugRetailAndUseApplyInfoMapper;


    @Override
    public DrugRetraceVO retrace(DrugRetraceSearcher searcher) {
        DrugRetraceVO drugRetraceVO = new DrugRetraceVO();
        //先查国内的，要是国内没有查询到，再查询国外的
        DrugProductionInfoVO drugProductionInfoVO = domesticDrugProductionApplyInfoMapper.selectByYpzsm(searcher.getYpzsm());
        if (Objects.isNull(drugProductionInfoVO)) {
            drugProductionInfoVO = drugImportApplyInfoMapper.selectByYpzsm(searcher.getYpzsm());
        }
        if (Objects.isNull(drugProductionInfoVO)) {
            throw new ServiceException("未查询到该药品！");
        }
        drugRetraceVO.setDrugProductionInfoVO(drugProductionInfoVO);
        //统一发货单与收货单，并行排序
        List<DrugInvoiceAndDeliveryInfoVO> drugInvoiceAndDeliveryInfoVOList = new ArrayList<>();
        // 查询发货单
        List<DrugInvoiceInfoVO> drugInvoiceInfoVOList = drugInvoiceApplyInfoMapper.selectByYpzsm(searcher.getYpzsm());
        if (!CollectionUtils.isEmpty(drugInvoiceInfoVOList)) {
            for (DrugInvoiceInfoVO drugInvoiceInfoVO : drugInvoiceInfoVOList) {
                DrugInvoiceAndDeliveryInfoVO drugInvoiceAndDeliveryInfoVO = new DrugInvoiceAndDeliveryInfoVO();
                drugInvoiceAndDeliveryInfoVO.setSj(drugInvoiceInfoVO.getFhsj());
                drugInvoiceAndDeliveryInfoVO.setTyshxydm(drugInvoiceInfoVO.getTyshxydmfhjg());
                drugInvoiceAndDeliveryInfoVO.setGsmc(drugInvoiceInfoVO.getFhjgmc());
                drugInvoiceAndDeliveryInfoVO.setDdh(drugInvoiceInfoVO.getFhdbh());
                drugInvoiceAndDeliveryInfoVO.setDjlx("invoice");
                drugInvoiceAndDeliveryInfoVOList.add(drugInvoiceAndDeliveryInfoVO);
            }
        }

        // 查询收货单
        List<DrugDeliveryNoteInfoVO> drugDeliveryNoteInfoVOList = drugDeliveryNoteApplyInfoMapper.selectByYpzsm(searcher.getYpzsm());
        if (!CollectionUtils.isEmpty(drugDeliveryNoteInfoVOList)) {
            for (DrugDeliveryNoteInfoVO drugDeliveryNoteInfoVO : drugDeliveryNoteInfoVOList) {
                DrugInvoiceAndDeliveryInfoVO drugInvoiceAndDeliveryInfoVO = new DrugInvoiceAndDeliveryInfoVO();
                drugInvoiceAndDeliveryInfoVO.setSj(drugDeliveryNoteInfoVO.getShsj());
                drugInvoiceAndDeliveryInfoVO.setTyshxydm(drugDeliveryNoteInfoVO.getTyshxydmshjg());
                drugInvoiceAndDeliveryInfoVO.setGsmc(drugDeliveryNoteInfoVO.getShjgmc());
                drugInvoiceAndDeliveryInfoVO.setDdh(drugDeliveryNoteInfoVO.getShdbh());
                drugInvoiceAndDeliveryInfoVO.setDjlx("delivery");
                drugInvoiceAndDeliveryInfoVOList.add(drugInvoiceAndDeliveryInfoVO);
            }
        }
        if (!CollectionUtils.isEmpty(drugInvoiceAndDeliveryInfoVOList)) {
            drugInvoiceAndDeliveryInfoVOList.sort((o1, o2) -> o1.getSj().compareTo(o2.getSj()));
        }
        drugRetraceVO.setDrugInvoiceAndDeliveryInfoVOList(drugInvoiceAndDeliveryInfoVOList);

        // 查询零售及使用信息
        DrugRetailAndUseInfoVO drugRetailAndUseInfoVO = drugRetailAndUseApplyInfoMapper.selectByYpzsm(searcher.getYpzsm());
        drugRetraceVO.setDrugRetailAndUseInfoVO(drugRetailAndUseInfoVO);

        return drugRetraceVO;
    }
}
